REML_core <- function(lm_bestmodel,         # Corresponding linear model
                      vn_zbar,              # Observations
                      li_settings_REML,     # Sets max min bounds for phi, and max # iterations
                      mn_initial_pop = NULL,# Eventual start value(s) for phi
                      mn_x0 = NULL          # Prediction design matrix (including column of ones);
                                            # if NULL, no prediction
                                )
{
  
  
  ## Performs REML estimation of phi / a, and sigma^2 / c1, and eventually universial 
  # kriging prediction
  # See also Brus et al., 2018, https://doi.org/10.1016/j.eja.2018.05.003
  
  # note: DMats.a2a and - for prediction DMats.a2p
  # should be in the global environment
  
  
  
  ## for testing purposes
  # li_settings_REML <- li_general_settings$li_case_assess_REML
  
  zbar <- vn_zbar
  
  vn_a_bounds                  <- li_settings_REML$vn_a_bounds
  n_max_iter                   <- li_settings_REML$n_max_iter
 


  #### Parameter estimation ####
 
  
  #Design-matrix for areas
  Xc <- model.matrix(lm_bestmodel)
  
  ## References to equations in the following:
  # Bea18 -> Brus et al, 2018, 
  # Geostatistical disaggregation of polygon maps of average crop yields by area-to-point kriging
  # doi.org/10.1016/j.eja.2018.05.003
  
  # Oea12 -> Orton et al, 2012
  # Spatial prediction of soil organic carbon from data on large and variable spatial supports. I.Inventory and mapping
  # doi.org/10.1002/env.2136
  
  
  #### Function to be optimised with respect to theta ####
  ## "negative residual log likelihood"
  ## If stated, returns other variables
  negresloglikelihood <- function(theta , returnAll = FALSE) {

    c1 = 1
    a = theta
    
    ## use the postParams code to calculate average covariance
    # 
    Ac <- averageCov(covModel = 'ExponentialC' , covPars = c(c1 , a) , DMats = DMats.a2a)
    
    #apply(Ac, as.list(vn_area_ids_for_obs_grid), FUN = mean)
    
    #print(Ac)
    
    eigval    <- eigen(Ac,only.values=TRUE)
    mineigval <- min(eigval$values)
    if (mineigval > 0) {
      AX<-solve(Ac,Xc)    
      XAX<- crossprod(Xc,AX)
      invXAX<-solve(XAX)
      Tc<-solve(Ac) - AX%*%invXAX%*%t(AX) # Bea18 Eq. 16, with T_{BB} -> Tc, C_{BB} -> Ac etc
      zTcz<-crossprod(zbar,Tc)%*%zbar
      sigma2 <- as.numeric(zTcz/(nrow(Xc)-ncol(Xc))) #Bea18 Eq. 17, Oea12 Eq 17
      Cc <- sigma2*Ac
      XCX <- XAX/sigma2
      logDetCc <- determinant(x = Cc, logarithm = TRUE)$modulus
      logDetXCX <- determinant(x = XCX, logarithm = TRUE)$modulus
      logLik<- as.numeric(-0.5 * (logDetCc + logDetXCX + zTcz/sigma2)) #ignores constant
      logLik<- -logLik } else 
      {logLik<-NA}
    if(returnAll){
      list('betahat'  = invXAX%*%t(AX)%*%zbar, #Bea18 Eq. 19; sigma2 cancels out
           'vbetahat' = invXAX* sigma2,                #Bea18 Eq. 20
           'sigma2'   = sigma2,
           'Ac'       = Ac,
           'Cc'       = Cc,
           'Tc'       = Tc)    
    }else{
      logLik
    }
  } # END negresloglikelihood <- function()
  
  
  #### Optimise ####
  
  lbound <- c(a = min(vn_a_bounds) )
  ubound <- c(a = max(vn_a_bounds) )
  optPars <- DEoptim(
    fn = negresloglikelihood,
    lower = lbound,
    upper = ubound,
    control = DEoptim.control(strategy = 2, 
                              bs       = F, 
                              NP       = 30, 
                              itermax  = n_max_iter, 
                              CR       = 0.5, 
                              F        = 0.8, 
                              trace    = 5,
                              initialpop = mn_initial_pop
    )
  )      
  
  n_a <- optPars$optim$bestmem
  
  #n_a <- 21100
  
  li_params <- negresloglikelihood(theta = n_a, 
                                   returnAll = TRUE) 
  
  

  n_c1 <- li_params$sigma2
  vn_betas <- as.numeric(li_params$betahat)
  names(vn_betas) <- rownames(li_params$betahat)
  vn_se_betas <- as.numeric(
                    sqrt(
                      diag(li_params$vbetahat)
                      )
                    )# Bea10 Eq. 20
  names(vn_se_betas) <- names(diag(li_params$vbetahat))
  
  # li_params$Ac[1:5, 1:5]
  # 
  # vn_betas
  # vn_se_betas
  # n_c1
  
  
  
  
  if ( is.null(mn_x0)  )
  {
    vn_pred <- NA
    vn_sd   <- NA
    
  } else #### Predict ####
  {
      
      n_predictions <- nrow(mn_x0)
    
      Cc <- averageCov(covModel = 'ExponentialC' , covPars = c(n_c1 , n_a ) , DMats = DMats.a2a)
      
      #Cc <- li_params$Cc  # the varcovar matrix, C bar _{BB} in Bea18 Eq. 13
      
      CX<-solve(Cc,Xc)       # CX = inv(C bar _{BB})   X bar _B 
      XCX<- crossprod(Xc,CX) # XCX = (X bar _B)^T  inv(C bar _{BB})   X bar _B 
      
      invXCX<-solve(XCX)     
      
      
      Tc<-solve(Cc) - CX%*%invXCX%*%t(CX) # Bea10 Eq. 16
      
      Tc <- li_params$Tc
      
      zTcz<-crossprod(zbar,Tc)%*%zbar
      #n_c1 <- as.numeric(zTcz/(nrow(Xc)-ncol(Xc))) 
      # see Eq. 17, paper Tom, https://doi.org/10.1002/env.2136
      # and Bea10 Eq. 17
      
      # n_a <- 4
      # n_c1 <- 2.7
      
      #compute betas (fixed effects)
      Cz  <-solve(Cc,zbar)    
      XCz <- crossprod(Xc,Cz)
      #vn_betas <- as.numeric(invXCX %*% XCz) # Bea10 Eq. 19
      XX  <- crossprod(Xc,Xc)
      XXinv <- solve(XX)
      
      (vtrendiid_max = max(diag(Xc %*% XXinv %*% t(Xc))))
      #vn_se_betas <- sqrt(diag(invXCX)) # Bea10 Eq. 20
    
      XCXX <- solve(XCX, t(Xc))
      XXCXX <- Xc%*%XCXX
      (vtrend_max <- max(diag(XXCXX)))
      
      #H0 <- spDists(sampleall,spdf_ar,longlat=TRUE)
      #save(H0,file="H0Matrix.Rdata")
      #load(file="H0Matrix.Rdata")
      
      #dim(mn_pred_obs_distances)
      
      # meanCov<-function(H,sample,cvm) {
      #   C<-as(cvm,"function")(H)
      #   Cbar<-tapply(C,INDEX=as.factor(sample$area),FUN=mean)
      # } 
      # 
      
      # meanCov<-function(H) 
      # {
      #   vn_C <- spatial_covariance(c1 = n_c1, a = n_a, h = H, c0 = 0)
      #   Cbar <- tapply(vn_C,INDEX=as.factor(vn_area_ids_for_obs_grid),FUN=mean)
      # }
      # 
      # c0 = 0
      # 
                                                                  
      Ckc_all <- averageCov(covModel = 'ExponentialC' , covPars = c(n_c1 , n_a ) , DMats = DMats.a2p)
      # observation area to prediction point averaged covariance matrix
      
      c0 <- 0 #"Nugget"
      
      residuals <- zbar - c(Xc %*% vn_betas)  #Bea18, last part of Eq. 12
      
      zpred <- varzpred <- vtrend <- vtrend_iid <- numeric(length=n_predictions)
      for (i in 1: (n_predictions) ) {
        if ( (i %% 100) == 0 )
        {
          txtOut <- paste0('Processing prediction points ' , i , ' (...+99) of ', n_predictions); cat("\r",txtOut)
        }

        
        # predpnt <- df_data_points[i,]
        # 
        # h0 <- abs(vn_rastered_areas - predpnt$s1)
        # 
        #Ckc<-meanCov(h0,sample=df_data_points,cvm=cvm)
        #Ckc<-meanCov(h0)
        
        Ckc <- Ckc_all[,i] 
        #vector with average covariances between prediction node and the areas  
        
        Cc0 <- solve(Cc,Ckc) # inv(C bar _{BB})  C_{Bs_0}
        
        # if (i==10)
        #   browser()
        
        x0          <- mn_x0[i,] # vector with relevant covariates
    
        mu          <- as.numeric( x0 %*% vn_betas  )
        
        zpred[i]    <- mu + crossprod(Cc0, residuals) 
                        #Bea18, equation 12: first part -> mu, lamba -> Cco; part between brackets -> residuals
        x_a         <- x0 - crossprod(Xc,Cc0)
        vtrend[i]   <- crossprod(x_a,solve(XCX, x_a))
        varzpred[i] <- c0 + n_c1 - t(Ckc)%*%Cc0  + vtrend[i]
                       #Bea18, equation 14: Cov(0) -> co + n_c1;  C bar _{Bs_0} -> Ckc
    
      }
      txtOut <- paste0('Finished predicting'); cat("\n",txtOut,"\n")
    
      vn_pred <- zpred
      
      if ( any(varzpred<0) )
      {
       cat("\n Warning: at least one negative prediction variance! \n")
      }
      vn_sd   <- sqrt(ifelse(varzpred<0,0,varzpred))
      
      #summary(varzpred)
      
  } # END predict
  
  
  
  return(
    list(
     vn_betas = vn_betas,
     vn_se_betas = vn_se_betas,
     n_a   =  n_a,
     n_c1  =  n_c1,
     vn_pred = vn_pred,  
     vn_sd   = vn_sd  
    ) 
    
    
  )
    
 
  
}  
  
  
