require(statmod) # For functions such as gauss.quad

averageCov <- function(covModel , covPars , DMats , nu = 0.5){

#####################################################
### for method = 'lq' : legendre gauss quad, 
###     DMats is list with : quadPtsD , wFD , maxD + method
### for method = 'dp' : average of cov fn for discretiation points
###     DMats is list with : discPtsD[n , n , nSamples^2]  + method
### for method = 'ad' : evaluation of cov fn at average sep dist
###     DMats is list with : avD + method
### for method = 'lq.dp4diag' : legendre gauss quad, 
###               with discretization points used for diagonal elements
###     DMats is list with : quadPtsD , wFD , maxD , discPtsD[n , nSamples^2] + method
### for method = 'dp.direct' : average of cov fn for discretiation points,
###               useful for testing, if discPtsD[n , n , nSamples^2] too big to store
###     DMats is list with : discPts1 and discPts2 (if non-symm) + method
### for method = 'qfh' : average of cov fn for quantiles of f(h) 
###     DMats is list with : discPtsD[n , n , nQtls]  + method
###     
#####################################################
    c1 <- covPars[1]
    a <- covPars[2]

    if(is.element(DMats$method , c('lq' , 'lq.dp4diag'))){
######################################################
### this algorithm computes covariances 
### quadPtsD (n1,n2,nGQPts array) are the Gauss quad evaluation points
### wFD (n1,n2,nGQPts array) are the product of weights and cdf of sep distances 
### at the quad evaluation points
### uses a pre-integration by parts, wF inputted.
######################################################
      n1 <- dim(DMats$quadPtsD)[[1]]
      n2 <- dim(DMats$quadPtsD)[[2]]
      nGQPts <- dim(DMats$quadPtsD)[[3]]

#############################################
### to replace all actual zero sep dists by tot var...
### if Dmax is 0 then we have point support data. 
#############################################
      iDmaxEQ0 <- which(DMats$maxD==0)
      fnCallStr <- paste0(covModel , '(DMats$maxD , a , nu)')
      CovMax <- c1 * eval(parse(text=fnCallStr))
      fnCallStr <- paste0('d' , covModel , '_wrth(DMats$maxD , a , nu)')
      Cov <- matrix(0 , n1 , n2)
      for (iGQPts in 1:nGQPts){
        dCov <- c1 * eval(parse(text=fnCallStr))
        
        dCov[iDmaxEQ0] <- 0
        
        Cov <- Cov + DMats$wFD[,,iGQPts] * dCov
      }
      Cov <- CovMax - Cov

    }else if(DMats$method == 'dp'){

      fnCallStr <- paste0(covModel , '(DMats$discPtsD , a , nu)')

      Cov <- c1 * eval(parse(text=fnCallStr))
      Cov <- apply(Cov , MARGIN=c(1,2) , FUN=mean , na.rm = T)

    }else if(DMats$method == 'dp.direct'){
      na1 <- length(DMats$discPts1)
      if(!is.null(DMats$points2)){
        symm <- FALSE
        na2 <- length(DMats$points2)
      }else{
        if(is.null(DMats$discPts2)){
          symm <- TRUE
          na2 <- na1
        }else{
          na2 <- length(DMats$discPts2)
        }
      }

      Cov <- matrix(NA , na1 , na2)
      if(!is.null(DMats$points2)){
#        for (i in 1:na1){
#          Cov[i,] <- colMeans(spDists(DMats$discPts1[[i]] , DMats$points2 , longlat = F))
#        }

        fnCallStr <- paste0(covModel , '(DTEMP , a , nu)')
        for (i in 1:na1){
          DTEMP <- spDists(DMats$discPts1[[i]] , DMats$points2 , longlat = F)
          CovThis <- c1 * eval(parse(text=fnCallStr))
          Cov[i,] <- colMeans(CovThis)
        }


      }else{

        fnCallStr <- paste0(covModel , '(DTEMP , a , nu)')
        for (i in 1:na1){
          if (symm){ minj <- i }else{ minj <- 1 }
          for (j in minj:na2){
            if (symm){
              DTEMP <- as.vector(spDists(DMats$discPts1[[i]] , DMats$discPts1[[j]] , longlat = F))
            }else{
              DTEMP <- as.vector(spDists(DMats$discPts1[[i]] , DMats$discPts2[[j]] , longlat = F))
            }
            CovThis <- c1 * eval(parse(text=fnCallStr))
            
            Cov[i,j] <- mean(CovThis)
            if (symm & (i != j)){
              Cov[j,i] <- Cov[i,j]
            }else{}
          }
        }
      }

    }else if(DMats$method == 'ad'){
      fnCallStr <- paste0(covModel , '(DMats$avD , a , nu)')
      Cov <- c1 * eval(parse(text=fnCallStr))

    }else if(DMats$method == 'qfh'){

      fnCallStr <- paste0(covModel , '(DMats$discPtsD , a , nu)')

      Cov <- c1 * eval(parse(text=fnCallStr))
      Cov <- apply(Cov , MARGIN=c(1,2) , FUN=mean , na.rm = T)

    }else{
      stop('Error - enter permissible method!')
    }

    if (DMats$method == 'lq.dp4diag'){
### update diagonal elements...
      fnCallStr <- paste0(covModel , '(DMats$discPtsD , a , nu)')
      CovDiag <- c1 * eval(parse(text=fnCallStr))
      
      CovDiag <- rowMeans(CovDiag , na.rm = T)
      diag(Cov) <- CovDiag    
    }else{}

    return(Cov)
}

#######################################################
### gauus.quad from statmod library gives [-1 , 1]
### transform from [-1 , 1] to [a , b] based on...
### https://rosettacode.org/wiki/Numerical_integration/Gauss-Legendre_Quadrature
#######################################################
lgqab <- function(nQuadPts , a , b){
  tmp <- gauss.quad(nQuadPts , kind="legendre")
  na <- length(a)

  nodes <- 0.5 * (matrix(a + b, na , nQuadPts) + matrix(b - a, na , 1) %*% matrix(tmp$nodes , nrow = 1))
  weights <- 0.5 * matrix(b - a , ncol = 1) %*% tmp$weights

  return(list('nodes' = nodes , 'weights' = weights))
}

####################################################
### functions to wrap up the set up functions...
### enter either discPts (list of matrices/dfs) or areas (spdf), not both
####################################################
DMats4AverageCov.a2a <- function(areas1 = NULL , areas2 = NULL  , discPts1 = NULL , discPts2 = NULL , nQuadPts = 15 , nSamples = 100 , method = 'lq' , type = 'random'){

  #browser()
  if (is.null(discPts1)){
######################################################
### if 'areas2' missing, assumed same as 'areas1'.
### coordinates should be planar not spherical
######################################################
     na1 <- length(areas1) 
    if (is.null(areas2)){
      areas2 <- areas1
      symm <- TRUE
    }else{
      symm <- FALSE
    }
    na2 <- length(areas2) 

####################################################
### sim disc pts: coordinates should be planar not spherical
####################################################
    discPts1 <- simDiscPts(areas = areas1 , nSamples = nSamples , type = type)
    if(!symm){
      discPts2 <- simDiscPts(areas = areas2 , nSamples = nSamples , type = type)
    }else{
      discPts2 <- NULL
    }
  }else{
    if(!is.null(areas1)){ stop('If entering discPts, do not also enter areas!') }else{} 
    na1 <- length(discPts1) 
    if (is.null(discPts2)){
      symm <- TRUE
      na2 <- na1 
    }else{
      symm <- FALSE
      na2 <- length(discPts2) 
    }
  }

#####################################################
### use disc pts to calc required matrices...
#####################################################
  if(method == 'lq'){
    DMats <- calcQuadPtsAndWghts.a2a(discPts1 = discPts1 , discPts2 = discPts2 , nQuadPts = nQuadPts)
  }else if (method == 'dp'){
    DMats <- calcDiscPtsD.a2a(discPts1 = discPts1 , discPts2 = discPts2)
  }else if (method == 'dp.direct'){
    DMats <- list('discPts1' = discPts1 , 'discPts2' = discPts2)
  }else if (method == 'ad'){
    DMats <- calcAvD.a2a(discPts1 = discPts1 , discPts2 = discPts2)
  }else if (method == 'lq.dp4diag'){
    DMats <- calcQuadPtsAndWghts.a2a(discPts1 = discPts1 , discPts2 = discPts2 , nQuadPts = nQuadPts)
    tmp <- calcDiscPtsD.a2a(discPts1 = discPts1 , discPts2 = discPts2 , diagOnly = TRUE)
    DMats$discPtsD <- tmp$discPtsD 
  }else if (method == 'qfh'){
    DMats <- calcDiscPtsD.a2a(discPts1 = discPts1 , discPts2 = discPts2)
    if(nQuadPts < dim(DMats$discPtsD)[3]){
      qPts <- seq(0 , 1 , 1 / (nQuadPts-1))
      qfhPts <- NA * DMats$discPtsD[,,1:nQuadPts]
      for (ia1 in 1:na1){ for (ia2 in 1:na2){ qfhPts[ia1,ia2,] <- quantile(DMats$discPtsD[ia1,ia2,] , qPts) } } 
      DMats$discPtsD <- qfhPts
    }else{} # else, just use the disc pts themselves.

  }else{
    stop('Error - enter valid method name!')
  }
  DMats$method <- method
  
  return(DMats)
}

DMats4AverageCov.a2p <- function(areas1 = NULL , points2 = NULL , discPts1 = NULL , nQuadPts = 15 , nSamples = 100 , method = 'lq' , type = 'random'){

  if (is.null(discPts1)){
####################################################
### sim disc pts: coordinates should be planar not spherical
####################################################
    discPts1 <- simDiscPts(areas = areas1 , nSamples = nSamples , type = type)
  }else{
    if(!is.null(areas1)){ stop('If entering discPts, do not also enter areas!') }else{} 
  }

#####################################################
### use disc pts to calc required matrices...
#####################################################
  if(method == 'lq'){
    DMats <- calcQuadPtsAndWghts.a2p(discPts1 = discPts1 , points2 = points2 , nQuadPts = nQuadPts)
  }else if (method == 'dp'){
    DMats <- calcDiscPtsD.a2p(discPts1 = discPts1 , points2 = points2)
  }else if (method == 'dp.direct'){
#    DMats <- list('discPts1' = discPts1) # not sure why I didn't have points2 in DMats
    DMats <- list('discPts1' = discPts1 , 'points2' = points2)
  }else if (method == 'ad'){
    DMats <- calcAvD.a2p(discPts1 = discPts1 , points2 = points2)
  }else if (method == 'lq.dp4diag'){
    stop('The lq.dp4diag method is only appropriate for symmetric matrices')
  }else if (method == 'qfh'){
    DMats <- calcDiscPtsD.a2p(discPts1 = discPts1 , points2 = points2)
    if(nQuadPts < dim(DMats$discPtsD)[3]){
      qPts <- seq(0 , 1 , 1 / (nQuadPts-1))
      qfhPts <- NA * DMats$discPtsD[,,1:nQuadPts]
      for (ia1 in 1:na1){ for (ia2 in 1:na2){ qfhPts[ia1,ia2,] <- quantile(DMats$discPtsD[ia1,ia2,] , qPts) } } 
      DMats$discPtsD <- qfhPts
    }else{} # else, just use the disc pts themselves.
    
  }else{
    stop('Error - enter valid method name!')
  }
  DMats$method <- method
  
  return(DMats)
}

#############################################################
### function to simulate the discretization points...
### coordinates should be planar not spherical
#############################################################
simDiscPts <- function(areas = NULL , nSamples = 100 , type = 'random'){
    na <- length(areas) 
    cDiscaList <- list() # list only used if synneteric.
    for (i in 1:na){
      cDiscaList[[i]] <- spsample(areas[i,] , nSamples , type = type)
    }
    return(cDiscaList)
}

#############################################################
### function to calculate the quadrature points...
#############################################################
calcQuadPtsAndWghts.a2a <- function(discPts1 , discPts2 , nQuadPts = 7){

    na1 <- length(discPts1) 
    if (is.null(discPts2)){
        symm <- TRUE
        na2 <- length(discPts1) 
    }else{
        symm <- FALSE
        na2 <- length(discPts2) 
    }
 
#############################
### a2a quadpts...
#############################
    DMats <- list()
    DMats$quadPtsD <- array(dim = c(na1 , na2 , nQuadPts))
    DMats$wFD <- array(dim = c(na1 , na2 , nQuadPts))
    DMats$maxD <- matrix(NA , na1 , na2)
    for (i in 1:na1){
      if (symm){ minj <- i }else{ minj <- 1 }
      for (j in minj:na2){
        if (symm){
          DTEMP <- as.vector(spDists(discPts1[[i]] , discPts1[[j]] , longlat = F))
        }else{
          DTEMP <- as.vector(spDists(discPts1[[i]] , discPts2[[j]] , longlat = F))
        }
        a <- min(DTEMP)
        b <- max(DTEMP)
        DMats$maxD[i,j] <- b

        tmp <- lgqab(nQuadPts,a,b)
        quadPtsTEMP <- tmp$nodes
        wTEMP <- tmp$weights

        nTEMP <- length(DTEMP)
        DMats$quadPtsD[i,j,] <- quadPtsTEMP
        FkdTEMP <- matrix(NA , 1 , nQuadPts)
        for (iQP in 1:nQuadPts){
            FkdTEMP[iQP] <- length(which(DTEMP <= quadPtsTEMP[iQP]))/nTEMP
        }
        DMats$wFD[i,j,] <- wTEMP * FkdTEMP
        if (symm & (i != j)){
            DMats$quadPtsD[j,i,] <- DMats$quadPtsD[i,j,]
            DMats$wFD[j,i,] <- DMats$wFD[i,j,]
            DMats$maxD[j,i] <- DMats$maxD[i,j]
        }else{}
      }
    }

    return(DMats)
}

#############################################################
### function to calculate the quadrature points...
#############################################################
calcQuadPtsAndWghts.a2p <- function(discPts1 , points2 , nQuadPts = 7){

    na1 <- length(discPts1) 
    np2 <- length(points2) 

#############################
### a2p quadpts...
#############################
    DMats <- list()
    DMats$quadPtsD <- array(dim = c(na1 , np2 , nQuadPts))
    DMats$wFD <- array(dim = c(na1 , np2 , nQuadPts))
    DMats$maxD <- matrix(NA , na1 , np2)
    for (i in 1:na1){
      DTEMP <- spDists(discPts1[[i]], points2 , longlat = F) 
      a <- apply(DTEMP , 2 , min)
      b <- apply(DTEMP , 2 , max)
      DMats$maxD[i,] <- b

      tmp <- lgqab(nQuadPts,a,b)
      quadPtsTEMP <- tmp$nodes
      wTEMP <- tmp$weights
        
      nTEMP <- nrow(DTEMP)
      DMats$quadPtsD[i,,] <- quadPtsTEMP
      FkdTEMP <- matrix(NA , np2 , nQuadPts)
      for (iQP in 1:nQuadPts){
          FkdTEMP[,iQP] <- colSums((DTEMP - matrix(quadPtsTEMP[,iQP], nTEMP , np2 , byrow = T))<0) / nTEMP
### long way...
#          for (jkl in 1:np2){
#            FkdTEMP[jkl,iQP] <- length(which(DTEMP[,jkl] <= quadPtsTEMP[jkl,iQP]))/nTEMP
#          }
      }

      DMats$wFD[i,,] <- wTEMP * FkdTEMP
    }

    return(DMats)
}

#############################################################
### function to get the separation distances of simulated discretiztion points...
#############################################################
calcDiscPtsD.a2a <- function(discPts1 , discPts2 , diagOnly = FALSE){

    na1 <- length(discPts1) 
    nSamples1Vec <- unlist(lapply(discPts1 , length))
    if (is.null(discPts2)){
        symm <- TRUE
        na2 <- length(discPts1) 
	  nSamples2Vec <- nSamples1Vec
    }else{
        if(diagOnly){ stop('diagOnly option only makes sense for symmetric matrices!') }else{}
        symm <- FALSE
        na2 <- length(discPts2) 
        nSamples2Vec <- unlist(lapply(discPts2 , length))
    }

####################################################
### use disc pts to calc vector of separation distances for each pair of areas...
### note if stratified sampling used to generate disc pts, then nSamples only approx
### so make matrix bigger, nSamplesVec gives variable lengths.  
####################################################
    DMats <- list()
    if (diagOnly){
      DMats$discPtsD <- matrix(NA , na1 , max(nSamples1Vec ^ 2))
    }else{
      DMats$discPtsD <- array(dim = c(na1 , na2 , max(nSamples1Vec) * max (nSamples2Vec)))
    }

    for (i in 1:na1){
      if (symm){ minj <- i }else{ minj <- 1 }
      if(diagOnly){ maxj <- i }else{ maxj <- na2 }
      for (j in minj:maxj){
        if(symm){
          if(diagOnly){
            vTmp <- as.vector(spDists(discPts1[[i]] , discPts1[[j]] , longlat = F))
            DMats$discPtsD[i,1:length(vTmp)] <- vTmp
          }else{
            vTmp <- as.vector(spDists(discPts1[[i]] , discPts1[[j]] , longlat = F))
            DMats$discPtsD[i,j,1:length(vTmp)] <- vTmp
          }
        }else{
          vTmp <- as.vector(spDists(discPts1[[i]] , discPts2[[j]] , longlat = F))
          DMats$discPtsD[i,j,1:length(vTmp)] <- vTmp
        }
        if (symm & (i != j)){
            DMats$discPtsD[j,i,] <- DMats$discPtsD[i,j,]
        }else{}
      }
    }

    return(DMats)
}

#############################################################
### function to calculate the quadrature points...
#############################################################
calcDiscPtsD.a2p <- function(discPts1  , points2){

    na1 <- length(discPts1) 
    np2 <- length(points2) 

    nSamplesVec <- unlist(lapply(discPts1 , length))

    DMats <- list()
    DMats$discPtsD <- array(dim = c(na1 , np2 , max(nSamplesVec)))
    for (i in 1:na1){
      DTEMP <- t(spDists(discPts1[[i]] , points2 , longlat = F))
      DMats$discPtsD[i,,1:nSamplesVec[i]] <- DTEMP
    }

    return(DMats)
}


#############################################################
### function to get the average separation distances of simulated discretiztion points...
#############################################################
calcAvD.a2a <- function(discPts1 , discPts2){

    na1 <- length(discPts1) 
    if (is.null(discPts2)){
        symm <- TRUE
        na2 <- length(discPts1) 
    }else{
        symm <- FALSE
        na2 <- length(discPts2) 
    }

####################################################
### use disc pts to calc matrix of average separation distances...
####################################################
    DMats <- list()
    DMats$avD <- matrix(NA , na1 , na2)
    for (i in 1:na1){
      if (symm){ minj <- i }else{ minj <- 1 }
      for (j in minj:na2){
        if(symm){
          DTEMP <- as.vector(spDists(discPts1[[i]] , discPts1[[j]] , longlat = F))
        }else{
          DTEMP <- as.vector(spDists(discPts1[[i]] , discPts2[[j]] , longlat = F))
        }
        DMats$avD[i,j] <- mean(DTEMP)
        if (symm & (i != j)){
            DMats$avD[j,i] <- DMats$avD[i,j]
        }else{}
      }
    }

    return(DMats)
}

#############################################################
### function to get the average separation distances of simulated discretiztion points...
#############################################################
calcAvD.a2p <- function(discPts1 , points2){

######################################################
### coordinates should be planar not spherical
######################################################
    na1 <- length(discPts1) 
    np2 <- length(points2) 

#############################
### a2p quadpts...
#############################
    DMats <- list()
    DMats$avD <- matrix(NA , na1 , np2)
    for (i in 1:na1){
      DMats$avD[i,] <- colMeans(spDists(discPts1[[i]] , points2 , longlat = F))
    }

    return(DMats)
}

#################################################################
### function to make a set of prediction blocks in prj units...
#################################################################
setPredictionBlock <- function(xCentre , yCentre , blockSize , prj){
  data <- data.frame(f=99.9)
  block0 <- cbind(c(0,0,blockSize , blockSize,0) ,c(0,blockSize , blockSize,0,0))
  block0 <- block0 - 0.5 * blockSize
  blockPrjList <- list() 
  for(i in 1:length(xCentre)){
    blockPrj <- block0
    blockPrj[,1] <- block0[,1] + xCentre[i] 
    blockPrj[,2] <- block0[,2] + yCentre[i]  
    blockPrj <- Polygon(blockPrj)
    blockPrj <- Polygons(list(blockPrj),1)
    blockPrj <- SpatialPolygons(list(blockPrj))
    proj4string(blockPrj) <- prj
    blockPrj <- SpatialPolygonsDataFrame(blockPrj,data)
    blockPrj <- spChFIDs(blockPrj, as.character(i))
    if(i == 1){
      blockPrjAll <- blockPrj
    }else{
      blockPrjAll <- rbind(blockPrjAll , blockPrj , makeUniqueIDs = TRUE)
    }
  }
  return(blockPrjAll)
}


###################################################################
### some correlation functions (ie variance = 1) and derivatives wrt par...
###################################################################
##############################################
### the exponential correlation function...
##############################################
ExponentialC <- function(h , par , nu = 0.5){
  return(exp(-h / par))
}

dExponentialC_wrta <- function(h , par , nu = 0.5){
  return(h * exp(-h / par) / (par ^ 2))
}

dExponentialC_wrth <- function(h , par , nu = 0.5){
  return( -exp(-h / par) / par)
}

##############################################
### the matern correlation function...
##############################################
MaternC <- function(h , par , nu){

    if((par > 0) & (nu >= 0.05)  & (nu <= 20)){
		ih0 <- which(h == 0)
		ihGT0 <- which(h > 0)

### range is approx rho * 3...this is from wiki, 
### and is i think what stein's parameterization was supposed to be.
		hsqrt2nuOVERa <- h * sqrt(2 * nu) / par 

		bes <- 0 * h
		bes[ihGT0] <- besselK(hsqrt2nuOVERa[ihGT0] , nu) 
      
		lnconstmatern <- NA * hsqrt2nuOVERa
	    lnconstmatern[ihGT0] <- nu * log(hsqrt2nuOVERa[ihGT0]) - (nu - 1) * log(2) - lgamma(nu)
      
		realmin <- 3.448490e-304 
		ibesGT0 <- which(bes > realmin)

        C <- 0 * h # initiate.
        C[ibesGT0] <- exp(lnconstmatern[ibesGT0]+log(bes[ibesGT0]))
        C[ih0] <- 1
        C[which(is.infinite(bes))] <- 1

	}else{
		C <- NA
	}

	return(C)
}

dMaternC_wrta <- function(h , par , nu){

    if((par > 0) & (nu >= 0.05)  & (nu <= 20)){
		ih0 <- which(h == 0)
		ihGT0 <- which(h > 0)

### range is approx rho * 3...this is from wiki, 
### and is i think what stein's parameterization was supposed to be.
		hsqrt2nuOVERa <- h * sqrt(2 * nu) / par 

		bes <- 0 * h
		bes[ihGT0] <- besselK(hsqrt2nuOVERa[ihGT0] , nu - 1) 
      
		lnconstmatern <- NA * hsqrt2nuOVERa
	    lnconstmatern[ihGT0] <- (nu + 1) * log(hsqrt2nuOVERa[ihGT0]) - (nu - 1) * log(2) - lgamma(nu) - log(par)
      
		realmin <- 3.448490e-304 
		ibesGT0 <- which(bes > realmin)

        C <- 0 * h # initiate.
        C[ibesGT0] <- exp(lnconstmatern[ibesGT0]+log(bes[ibesGT0]))
        if(length(ih0) > 0){
          minPosh <- min(h[ihGT0])
          hSmall <- 1E-10
          if(hSmall > minPosh){ stop('Define appropriate small h for this dataset in dMaternC_wrta!') }else{}
          hSmallsqrt2nuOVERa <- hSmall * sqrt(2 * nu) / par 
  		  besSmall <- besselK(hSmallsqrt2nuOVERa , nu - 1) 
  	      lnconstmatern <- (nu + 1) * log(hSmallsqrt2nuOVERa) - (nu - 1) * log(2) - lgamma(nu) - log(par)
          derivhSmall <- -(exp(lnconstmatern+log(besSmall)))

          C[ih0] <- derivhSmall

        }else{}

        C[which(is.infinite(bes))] <- NA

	}else{
		C <- NA
	}

	return(C)
}

dMaternC_wrth <- function(h , par , nu){

    if((par > 0) & (nu >= 0.05)  & (nu <= 20)){
		ih0 <- which(h == 0)
		ihGT0 <- which(h > 0)

### range is approx rho * 3...this is from wiki, 
### and is i think what stein's parameterization was supposed to be.
		hsqrt2nuOVERa <- h * sqrt(2 * nu) / par 

		bes <- 0 * h
		bes[ihGT0] <- besselK(hsqrt2nuOVERa[ihGT0] , nu - 1) 
      
		lnconstmatern <- NA * hsqrt2nuOVERa
	    lnconstmatern[ihGT0] <- nu * log(hsqrt2nuOVERa[ihGT0]) - (nu - 1) * log(2) - lgamma(nu) + log(sqrt(2 * nu) / par)
      
		realmin <- 3.448490e-304 
		ibesGT0 <- which(bes > realmin)

        C <- 0 * h # initiate.
        C[ibesGT0] <- -(exp(lnconstmatern[ibesGT0]+log(bes[ibesGT0])))
        
        if(length(ih0) > 0){
          minPosh <- min(h[ihGT0])
          hSmall <- 1E-10
          if(hSmall > minPosh){ stop('Define appropriate small h for this dataset in dMaternC_wrth!') }else{}
          hSmallsqrt2nuOVERa <- hSmall * sqrt(2 * nu) / par 
  		  besSmall <- besselK(hSmallsqrt2nuOVERa , nu - 1) 
  	      lnconstmatern <- nu * log(hSmallsqrt2nuOVERa) - (nu - 1) * log(2) - lgamma(nu) + log(sqrt(2 * nu) / par)
          derivhSmall <- -(exp(lnconstmatern+log(besSmall)))

          C[ih0] <- derivhSmall

        }else{}

        C[which(is.infinite(bes))] <- NA

	}else{
		C <- NA
	}

	return(C)
}


### so that lq works with ref prior to calculate average of dAda....
ddExponentialC_wrta_wrth <- function(h , par , nu) {
  return( (1 - h) * exp(-h / par) / (par ^ 2) )
}

