load_case_data <- function(li_general_settings,
                                   n_prediction_thinning = 1  # For testing purposes
                                   ) 
  
## Loads the observation data, the covariates and the prediction mask
{
  
  # Gets  the data from li_general_settings$str_path_to_obs_data

  # More details about the dataset can be found at Brus et al., 2018, 
  # Geostatistical disaggregation of polygon maps of average crop yields 
  # by area-to-point kriging, European Journal of Agronomy, Volume 97, 
  # July 2018, Pages 48-59; https://doi.org/10.1016/j.eja.2018.05.003
  
  #### Load yields and covariates per area ####
  # The aggregated yields of sorghum and millet for the 45 regions in Burkina Faso are read. 

  df_yields_raw <- read.csv( 
    paste0(li_general_settings$str_path_to_obs_data,
           "Data_Burkina_Faso_improved_yield_stats.csv")
  )
  
  # From kg/ha to tonnes/ha
  df_yields_raw$AvgOfOFFICIAL_YIELD <- df_yields_raw$AvgOfOFFICIAL_YIELD/1000 
  
  # From long to wide format
  df_yields <- reshape(df_yields_raw, 
                  idvar = c("REG_MAP_ID", "REG_NAME"), 
                  timevar = "STAT_CROP_NAME", 
                  drop = "STAT_CROP_NO",
                  direction = "wide")
  remove(df_yields_raw)
  
  names(df_yields)[c(3,4)] <- c("sorghum", "millet")
  
  # Take care that order is everywhere the same
  
  vn_ord <- order(df_yields$REG_MAP_ID)
  
  df_yields <- df_yields[vn_ord,]
  
  remove(vn_ord)
  
  attr(df_yields,'description') <-
       "Dataframe with yields as mean per ares"
  
  #### Add geographical information and prediction grid ####
  # Read the GIS file with the areas.
  
  # read shape file (including projection attributes) of provinces
  spoldf_areas <- readOGR(dsn   =  sub(pattern = "/$",
                                       replacement = "",
                                       x = li_general_settings$str_path_to_obs_data), # Remove trailing slash
                          layer = "BFA_adm2")
  names(spoldf_areas)[15] <- "REG_MAP_ID" # instead of "REG_MAP"
  attr(spoldf_areas,'description') <-
    "Spatial polygone data frame, provinces of Burkina Faso"
  
  #spplot(spoldf_areas , zcol="REG_MAP_ID")
  str_prj <- proj4string(spoldf_areas )
  # cat(str_prj)
  # browser()
  attr(str_prj,'description') <- "String; projection for all spatial objects"
  
  # read boxshaped raster, store resolution and take out research area (provinces) of interest
  # also select where crops are grown
  r_box <- raster( 
    paste0(li_general_settings$str_path_to_obs_data,
           "BurkinaFaso_mask.bil" )
  )
  n_raster_resolution <- res(r_box)[1] 
  #attr(str_prj,'n_raster_resolution') <- "Scalar; resolution of prediction raster "
  
  spxdf_all <- as(r_box, "SpatialPixelsDataFrame")
  remove(r_box)
  
  vn_identifiers_province <- which(spxdf_all$BurkinaFaso_mask==2)
  spxdf_study_area <- spxdf_all[vn_identifiers_province,] 
  
  spdf_study_area <- as(spxdf_study_area, "SpatialPointsDataFrame")
  proj4string(spdf_study_area) <- str_prj
  
  ## Create dataframe with only relevant gridpoints in country
  df_areas_properties_per_point <- (spdf_study_area %over% spoldf_areas )
  df_ar <- as(spdf_study_area,"data.frame")
  df_ar$REG_MAP_ID <- df_areas_properties_per_point$REG_MAP_ID
  df_ar <- df_ar[!is.na(df_ar$REG_MAP_ID),]
  
  remove(spxdf_all, df_areas_properties_per_point, vn_identifiers_province)
  
  # extend spoldf_areas data with the yields per area  
  spoldf_areas_yld<-merge(x = spoldf_areas,
                          y = df_yields[,c(1,3,4)]
                          )
  
  attr(spoldf_areas_yld,'description') <-
    "Spatial polygons data frame, showing provinces (areas) and yields per area"  
  

  ## thinning (for testing purposes)
  if (n_prediction_thinning>1)
  {
    df_ar <- df_ar[seq(from = 1, 
                       to = nrow(df_ar),
                       by = n_prediction_thinning),]  # prediction dilution factor
  }
  
  # Extract values of predictors at prediction grid, and scale the predictors ####
  
  spdf_pred_grid_with_cov <- df_ar; coordinates(spdf_pred_grid_with_cov) <- ~x+y
  
  
  spdf_pred_grid_with_cov <- d<-merge(x = spdf_pred_grid_with_cov,
                          y = df_yields[,c(1,3,4)]
  )
  
  #### Load covariates raster for prediction (both unscaled and scaled) ####
  
  # Covariate: CFAPAR_27
  
  rst <- raster( 
    paste0(li_general_settings$str_path_to_obs_data,
           "xcfapar_27.bil")
  )

  spdf_pred_grid_with_cov$CFAPAR_27 <- extract(rst,spdf_pred_grid_with_cov)
  spdf_pred_grid_with_cov$CFAPAR_27_scd <- scale(spdf_pred_grid_with_cov$CFAPAR_27)
  remove(rst)

  # Covariate: CRAIN_EC_27
  
  rst <- raster( 
    paste0(li_general_settings$str_path_to_obs_data,
           "xcrain_ec_27.bil")
  )
  
  spdf_pred_grid_with_cov$CRAIN_EC_27 <- extract(rst,spdf_pred_grid_with_cov)
  spdf_pred_grid_with_cov$CRAIN_EC_27_scd <- scale(spdf_pred_grid_with_cov$CRAIN_EC_27)
  remove(rst)
  

  # Covariate: TMIN_EC_21
  rst <- raster(  
    paste0(li_general_settings$str_path_to_obs_data,
           "TMIN_EC_21.bil")
  )
  spdf_pred_grid_with_cov$TMIN_EC_21 <- extract(rst,spdf_pred_grid_with_cov)
  spdf_pred_grid_with_cov$TMIN_EC_21_scd <- scale(spdf_pred_grid_with_cov$TMIN_EC_21)
  remove(rst)


  # Covariate: PHAQ10
  rst <- raster(  
    paste0(li_general_settings$str_path_to_obs_data,
           "PHAQ101.bil")
  )

  spdf_pred_grid_with_cov$PHAQ10 <- extract(rst,spdf_pred_grid_with_cov)
  spdf_pred_grid_with_cov$PHAQ10_scd <- scale(spdf_pred_grid_with_cov$PHAQ10)  
  remove(rst)  
  
  vb_complete <-  (complete.cases(spdf_pred_grid_with_cov@data))

  spdf_pred_grid_with_cov@coords <- spdf_pred_grid_with_cov@coords[vb_complete,]
  spdf_pred_grid_with_cov@data <- spdf_pred_grid_with_cov@data[vb_complete,]
  
  
  proj4string(spdf_pred_grid_with_cov)<-str_prj
  attr(spdf_pred_grid_with_cov,'description') <- "Spatial points dataframe with all gridpoints to predict of area of interest, 
  with covariates (scaled and unscaled), with coordinates and area ID. Contains also the observations (pointwise, but value is area mean)"

  
  ## Create 
  sp_pred_grid <- SpatialPoints(spdf_pred_grid_with_cov@coords, proj4string=CRS(str_prj))

  return(
    list(spdf_pred_grid_with_cov = spdf_pred_grid_with_cov,
                                  df_yields          = df_yields,
                                  spoldf_areas       = spoldf_areas,
                                  sp_pred_grid       = sp_pred_grid,
                                  str_prj            = str_prj
                                  )
        )  
  
}
