# Please see README.pdf 

## Performs analysis and prediction for the case data. In the beginning of the script, 
# one can choose the variable of interest (PAR as synthetic case study, millet and sorghum). 
# Produces Figures 6 and 7, and the figures in Online Resource F

## We strongly recommend to run this code on a computer with at least 16GB working memory
# and ample swapping space; even then it might take several hours to run

## For fast algorithm testing, we recommend to set the n_prediction_thinning argument in the 
# function call to load_case_data (section #### Load observation data Burkina Faso ####) to 50.


#### Goal ####

li_general_settings <- list()

## Choose here, by commenting in, one the variable of interest (voi) 
# out of the 3 options:

li_general_settings$str_case  <- "PAR"      # Synthetic case 
#li_general_settings$str_case  <- "millet"  # Figures 6 and 7 in paper 
#li_general_settings$str_case  <- "sorghum" # Figures in Online Resource F


li_general_settings$str_path_to_obs_data    <- "./data_burkina_faso/"

#### Packages and basic functions module ####
vs_needed_packages <- c("sp", 
                        "invgamma", 
                        "DEoptim", 
                        "rgdal",
                        "raster",
                        "grid",      
                        "lattice",
                        "statmod" 
)

source("setup.R")
setup(vs_needed_packages)
remove(vs_needed_packages)


#### Load observation data Burkina Faso ####
source("load_case_data.R")

li_obs_data_cov_predgrid <- load_case_data(li_general_settings,
                                           n_prediction_thinning = 1
                                           # for example 50 for testing purposes; 
                                           # speeds up enormously + lower memory requirement
                                           # set 1 for production purposes
                                           # > 50 gives problems because of
                                           # disappearing provinces
)  

#### Prepare variable of interest, lm model, and covariate prediction matrix ####
if (li_general_settings$str_case  == "PAR")
{
  spdf_grid_CFA_PAR_27 <- li_obs_data_cov_predgrid$spdf_pred_grid_with_cov
  
  vs_names <- names(spdf_grid_CFA_PAR_27)
  
  spdf_grid_CFA_PAR_27@data[,!(vs_names %in% c("REG_MAP_ID", "CFAPAR_27"))] <- NULL
  
  spplot(spdf_grid_CFA_PAR_27, zcol = "CFAPAR_27")
  
  remove(vs_names)
  
  # Average value per province
  vn_avg_CFA_PAR <- as.vector( 
    tapply(spdf_grid_CFA_PAR_27@data[,"CFAPAR_27"],
           INDEX = spdf_grid_CFA_PAR_27@data[,"REG_MAP_ID"],
           FUN = mean
    )
  )  
  
  vn_zbar <- vn_avg_CFA_PAR
  
  vn_reg_map_ids <- unique(spdf_grid_CFA_PAR_27@data[,"REG_MAP_ID"])
  
  #order
  vn_reg_map_ids <- vn_reg_map_ids[order(vn_reg_map_ids)]
  
  df_index_voi <- data.frame(REG_MAP_ID = vn_reg_map_ids,
                             voi = vn_zbar)
  
  li_obs_data_cov_predgrid$vn_zbar <- vn_zbar
  
  spdf_pred_grid_with_cov <- li_obs_data_cov_predgrid$spdf_pred_grid_with_cov
  
  spdf_pred_grid_with_cov <- merge(x = spdf_pred_grid_with_cov,
                                   y = df_index_voi)
  
  spplot(spdf_pred_grid_with_cov, zcol = "voi")
  
  li_obs_data_cov_predgrid$spdf_pred_grid_with_cov <- spdf_pred_grid_with_cov
  
  li_obs_data_cov_predgrid$spdf_pred_grid_with_cov$voi
  
  #### Create trend model, and predictions covariate matrix ####
  # create design matrix
  
  Xc <- matrix(NA, nrow = 45, ncol=3)
  colnames(Xc) <- c( "CRAIN_EC_27_scd", "TMIN_EC_21_scd", "PHAQ10_scd")
  for (i in c( "CRAIN_EC_27_scd", "TMIN_EC_21_scd", "PHAQ10_scd"))
  {  
    
    Xc[,i] <- tapply(X = li_obs_data_cov_predgrid$spdf_pred_grid_with_cov@data[,i],
                     INDEX = li_obs_data_cov_predgrid$spdf_pred_grid_with_cov$REG_MAP_ID,
                     FUN = mean,
                     na.rm = TRUE
    )
    
  }  
  remove(i)
  
  df_voi_and_covars <- data.frame(
    cbind(vn_zbar, Xc)
  )
  # 'voi': variable of interest
  
  names(df_voi_and_covars)[1] <- "voi"
  
  lm_bestmodel <- lm(voi ~ CRAIN_EC_27_scd+TMIN_EC_21_scd+PHAQ10_scd, 
                     data = df_voi_and_covars
  )
  
  ## covariance matrix for predictions
  mn_x0 <- cbind(1, as.matrix(li_obs_data_cov_predgrid$spdf_pred_grid_with_cov@data[, c(8,10,12)]) )  
  
  mn_x0_BAK <- mn_x0[,2:4] # The BAK algorithm expects a differently shaped covariate prediction matrix
  

  spdf_pred_grid_with_cov$ones <- 1
  
  vn_counts <-  tapply(X = spdf_pred_grid_with_cov$ones,
                       INDEX = spdf_pred_grid_with_cov$REG_MAP_ID,
                       FUN = sum,
                       na.rm = TRUE
  )
  
  vn_baseline <- rep(NA, sum(vn_counts))
  
  vn_cum_counts <- c(0, cumsum(vn_counts) )
  
  for (i in 1:45)
  {
    
    vn_baseline[(vn_cum_counts[i]+1): vn_cum_counts[i+1]]  <- vn_avg_CFA_PAR[i]
    
    #cat("\n", vn_cum_counts[i]+1, " ",vn_cum_counts[i+1])
    
  }
  
  li_obs_data_cov_predgrid$spdf_pred_grid_with_cov$baseline <- vn_baseline
  
  
  
} else if(li_general_settings$str_case  == "sorghum")
{  
  df_data_area            <- data.frame(
    li_obs_data_cov_predgrid$df_yields$sorghum
  )
  names(df_data_area) <- "yield"
  
  #vector with means of response variable for regions
  vn_zbar <- df_data_area$yield
  
  spdf_pred_grid_with_cov <- li_obs_data_cov_predgrid$spdf_pred_grid_with_cov        
  
  Xc <- as.matrix(
    data.frame(intercept       = seq(from = 1, to=1, length.out = 45), 
               CFAPAR_27_scd   = NA,
               CRAIN_EC_27_scd = NA, 
               TMIN_EC_21_scd  = NA,
               PHAQ10_scd      = NA)
  )
  
  
  # Fill the design matrix with the relevant covariates
  for (i in c("CFAPAR_27_scd", "CRAIN_EC_27_scd", "TMIN_EC_21_scd", "PHAQ10_scd"))
  {  
    
    Xc[,i] <- tapply(X = spdf_pred_grid_with_cov@data[,i],
                     INDEX = spdf_pred_grid_with_cov$REG_MAP_ID,
                     FUN = mean,
                     na.rm = TRUE
    )
    
  }  
  remove(i)
  
  df_voi_and_covars <- data.frame(
    cbind(vn_zbar, Xc)
  )
  # 'voi': variable of interest, in this case sorghum yield
  
  names(df_voi_and_covars)[1] <- "voi"
  
  
  lm_bestmodel <- lm(voi ~ CFAPAR_27_scd+CRAIN_EC_27_scd+TMIN_EC_21_scd+PHAQ10_scd, 
                     data = df_voi_and_covars
  )
  
  spdf_pred_grid_with_cov <- li_obs_data_cov_predgrid$spdf_pred_grid_with_cov
  spoldf_areas            <- li_obs_data_cov_predgrid$spoldf_areas
  
  ## covariance matrix for predictions
  mn_x0 <- cbind(1, as.matrix(li_obs_data_cov_predgrid$spdf_pred_grid_with_cov@data[, c(6, 8,10,12)]) )  
  
  mn_x0_BAK <- mn_x0[,2:5] # The BAK algorithm expects a differently shaped covariate prediction matrix    
  
  
  
} else if(li_general_settings$str_case == "millet")
{
  
  df_data_area            <- data.frame(
    li_obs_data_cov_predgrid$df_yields$millet
  )
  
  names(df_data_area) <- "yield"
  
  
  #vector with means of response variable for regions
  vn_zbar <- df_data_area$yield
  
  
  Xc <- as.matrix(
    data.frame( intercept       = seq(from = 1, to=1, length.out = 45) )
  )
  
  
  df_voi_and_covars <- data.frame(
    cbind(vn_zbar, Xc)
  )
  # 'voi': variable of interest, in this case millet yield
  
  names(df_voi_and_covars)[1] <- "voi"
  
  lm_bestmodel <- lm(voi ~ 1, 
                     data = df_voi_and_covars
  )
  
  spdf_pred_grid_with_cov <- li_obs_data_cov_predgrid$spdf_pred_grid_with_cov 
  spoldf_areas            <- li_obs_data_cov_predgrid$spoldf_areas
  
  ## covariance matrix for predictions
  mn_x0 <- matrix(1, nrow = length(li_obs_data_cov_predgrid$spdf_pred_grid_with_cov), ncol = 1)  
  
  mn_x0_BAK <- matrix(NA, nrow = length(li_obs_data_cov_predgrid$spdf_pred_grid_with_cov), ncol = 1)  
  # The BAK algorithm expects a differently shaped covariate prediction matrix
  
  
} else 
{
  stop('Error - enter a valid description of the dataset in li_general_settings$str_case')
}



#### Give all spatial objects a planar projection ####
str_prj_planar <- "+proj=robin +datum=WGS84"
spdf_pred_grid_with_cov_plan <- spTransform(li_obs_data_cov_predgrid$spdf_pred_grid_with_cov, CRS(str_prj_planar) )
spoldf_areas_plan <- spTransform(li_obs_data_cov_predgrid$spoldf_areas, CRS(str_prj_planar) )
sp_pred_grid_plan <- spTransform(li_obs_data_cov_predgrid$sp_pred_grid, CRS(str_prj_planar) )



#### Create covariance objects ####
## Create DMats.a2a and DMats.a2p objects in the general scope, 
## for later use  for average covariance calculations 
## This can take a while.
source("create_covariance_objects.R")
create_covariance_objects(spdf_pred_grid_with_cov_plan, 
                          sp_pred_grid_plan 
)




#### Estimate and predict using REML ####

#### General  settings REML estimation and prediction #
li_general_settings$li_case_assess_REML$vn_a_bounds          <- c(37E3 , 300E3)
li_general_settings$li_case_assess_REML$n_max_iter           <- 10

source( "REML_core.R" ) 
set.seed(271828) # REML optimalisation uses some randomness
li_REML_results <- REML_core(lm_bestmodel = lm_bestmodel,
                             vn_zbar      = vn_zbar,
                             li_settings_REML = li_general_settings$li_case_assess_REML,
                             mn_initial_pop =  NULL, #matrix(rnorm(30, 100E3, 1), nrow=30, ncol=1), # forcing start values
                             mn_x0 = mn_x0
)

# li_REML_results$vn_se_betas
# li_REML_results$n_c1
# li_REML_results$n_a

spdf_prediction_REML <- li_obs_data_cov_predgrid$spdf_pred_grid_with_cov
spdf_prediction_REML$prediction    <- li_REML_results$vn_pred
spdf_prediction_REML$prediction_sd <- li_REML_results$vn_sd


#### General settings for BAK ####
if(is.null(li_REML_results$n_c1))
{
  c1Vec <- seq(0.005 , 200 , 5)
} else
{
  c1Vec <- seq(li_REML_results$n_c1/25, li_REML_results$n_c1*1.5 , length.out = 30)
}

## set parameter range for c1 around the value estimated by REML
# (has only influence on marginal plot)

priorc1Pars <<- c(0,0)

k <- dim(Xc)[[2]]


Xc <- model.matrix(lm_bestmodel)  # BAK needs a column of ones


#### BAK reference ####
prioraType           <<- 'reference'
prioraPars           <<-  NA

aVec <- 
  c( seq(500 , 50E3 , length.out = 25) , 
     seq(50E3 , 150E3 , length.out = 10),
     seq(200E3 , 1000E3 , length.out = 10)   )



li_BAK_results_ref <- postc1abetaz0(z        = vn_zbar, 
                                    X        = Xc , 
                                    aVec     = aVec, 
                                    c1Vec    = c1Vec,
                                    #betaMat  = betaMat, # Default
                                    covsGrid = mn_x0_BAK
)

spdf_prediction_BAK_ref <- li_obs_data_cov_predgrid$spdf_pred_grid_with_cov
spdf_prediction_BAK_ref$prediction    <- li_BAK_results_ref$z0


if ( any(li_BAK_results_ref$v0<0) )
{
  cat("\n Warning: at least one negative prediction variance! \n")
}
spdf_prediction_BAK_ref$prediction_sd   <- sqrt(ifelse(li_BAK_results_ref$v0<0,0,li_BAK_results_ref$v0))


#### BAK uniform ####
prioraType           <<- 'uni'
prioraPars           <<- c(37E3-1, 300E3+1)

aVec <- c( seq(37E3 , 150E3 , length.out = 25), seq(151E3 , 300E3 , length.out = 5) )  


li_BAK_results_uni <- postc1abetaz0(z        = vn_zbar, 
                                    X        = Xc , 
                                    aVec     = aVec, 
                                    c1Vec    = c1Vec,
                                    #betaMat  = betaMat,
                                    covsGrid = mn_x0_BAK
)


spdf_prediction_BAK_uni <- li_obs_data_cov_predgrid$spdf_pred_grid_with_cov
spdf_prediction_BAK_uni$prediction    <- li_BAK_results_uni$z0


if ( any(li_BAK_results_uni$v0<0) )
{
  cat("\n Warning: at least one negative prediction variance! \n")
}
spdf_prediction_BAK_uni$prediction_sd   <- sqrt(ifelse(li_BAK_results_uni$v0<0,0,li_BAK_results_uni$v0))

#### MML ####

## posterior mode range parameter with uniform prior

n_mode_a <- li_BAK_results_uni$aVec[which.max(li_BAK_results_uni$fpa)]

prioraType   <<- 'uni'
prioraPars   <<- c( n_mode_a - 0.2, n_mode_a+0.2 )
aVec         <-  c( n_mode_a - 0.1, n_mode_a, n_mode_a+0.1 )  


li_mml_results <- postc1abetaz0(z        = vn_zbar, 
                                X        = Xc , 
                                aVec     = aVec, 
                                c1Vec    = c1Vec,
                                #betaMat  = betaMat,
                                covsGrid = mn_x0_BAK
)

spdf_prediction_MML <- li_obs_data_cov_predgrid$spdf_pred_grid_with_cov
spdf_prediction_MML$prediction    <- li_mml_results$z0


if ( any(li_mml_results$v0<0) )
{
  cat("\n Warning: at least one negative prediction variance! \n")
}
spdf_prediction_MML$prediction_sd   <- sqrt(ifelse(li_mml_results$v0<0,0,li_mml_results$v0))




#### General settings export plots ####
li_general_settings$li_export$bo_plots_to_latex  <- FALSE # as .pdf


bo_bw <- FALSE  # Black and white plots

if (bo_bw)
{  
  li_general_settings$li_export$latex_plot_dir     <-
    "./exportbw/"
} else
{
  li_general_settings$li_export$latex_plot_dir     <-
    "./exportcolour/"
}  
li_general_settings$li_export$use_greek     <- TRUE


#### Show parameter assessments ####
source( "show_parameter_assessments.R" )

## Uniform prior (partly used for Subfigures F4 a and b)
show_parameter_assessments(li_postParams = li_BAK_results_uni, 
                           li_estimations_and_predictions_REML = li_REML_results,
                           #li_general_settings$li_simulation_generate,
                           li_export = li_general_settings$li_export,
                           bo_show_marginals_postParams  = FALSE, # the build in function in postParams
                           bo_show_marginals  = TRUE, # plots adjusted for simulated case                                        
                           bo_show_joint_c1_a = FALSE,
                           bo_show_joint_beta_a = FALSE
)

## Reference prior (partly used for Subfigures F4 c and d)
show_parameter_assessments(li_postParams = li_BAK_results_ref, 
                           li_estimations_and_predictions_REML = li_REML_results,
                           #li_general_settings$li_simulation_generate,
                           li_export = li_general_settings$li_export,
                           bo_show_marginals_postParams  = FALSE, # the build in function in postParams
                           bo_show_marginals  = TRUE, # plots adjusted for simulated case                                        
                           bo_show_joint_c1_a = FALSE,
                           bo_show_joint_beta_a = FALSE
)




#### Show original and created maps ####
source("show_case_data.R") 

### Show map covariates; Figure F1 
show_case_data (li_yields_covars_predmask = li_obs_data_cov_predgrid,
                spdf_prediction   = NULL,
                li_export       = li_general_settings$li_export,
                bo_show_hist    = FALSE,  # histogram of yields
                bo_show_yields  = FALSE,   # map of yields/voi
                bo_show_covars  = TRUE,  # combined maps of selected covariates, in case of sorghum
                str_casestudy   = li_general_settings$str_case,    # sorghum, millet, PAR
                str_title_add   = NA    #  # for the prediction
)



### Show maps observations, prediction, prediction sd (for Subfigures 6a,b,c in case of millet)
show_case_data (li_yields_covars_predmask = li_obs_data_cov_predgrid,
                spdf_prediction   = spdf_prediction_MML ,
                li_export       = li_general_settings$li_export,
                bo_show_hist    = FALSE,  # histogram of yields
                bo_show_yields  = TRUE,   # map of yields/voi
                bo_show_covars  = FALSE,  # combined maps of selected covariates, in case of sorghum
                str_casestudy   = li_general_settings$str_case,    # sorghum, millet, PAR
                str_title_add   = " MML"    #  # for the prediction
)

### Show detail map (for Subfigures 6d,e,f in case of millet)
show_case_data (li_yields_covars_predmask = li_obs_data_cov_predgrid,
                spdf_prediction   = spdf_prediction_MML ,
                li_export       = li_general_settings$li_export,
                bo_show_hist    = FALSE,  # histogram of mean yields per area; not used in paper
                bo_show_yields  = TRUE,   # map of mean yields/voi per area
                bo_show_covars  = FALSE,  # combined maps of selected covariates, in case of sorghum
                str_casestudy   = li_general_settings$str_case,    # sorghum, millet, PAR
                str_title_add   = " MML",    #  # for the prediction
                vn_xlim         = c(-5.5,-2.25),       # in case we want a detail area
                vn_ylim         = c(10, 12),
                bo_is_detail    = TRUE
)
#


### Show maps observations, prediction, prediction sd (for Subfigures F2a,b,c in case of sorghum)
show_case_data (li_yields_covars_predmask = li_obs_data_cov_predgrid,
                spdf_prediction   = spdf_prediction_BAK_ref ,
                li_export       = li_general_settings$li_export,
                bo_show_hist    = FALSE,  # histogram of yields
                bo_show_yields  = TRUE,   # map of yields/voi
                bo_show_covars  = FALSE,  # combined maps of selected covariates, in case of sorghum
                str_casestudy   = li_general_settings$str_case,    # sorghum, millet, CFA_PAR
                str_title_add   = " ref prior"    #  # for the prediction
)

### Show maps observations, prediction, prediction sd (for Subfigures F2 d and e  in case of sorghum)
show_case_data (li_yields_covars_predmask = li_obs_data_cov_predgrid,
                spdf_prediction   = spdf_prediction_BAK_ref ,
                li_export       = li_general_settings$li_export,
                bo_show_hist    = FALSE,  # histogram of yields
                bo_show_yields  = FALSE,  # map of yields/voi
                bo_show_covars  = FALSE,  # combined maps of selected covariates, in case of sorghum
                str_casestudy   = li_general_settings$str_case,    # sorghum, millet, CFA_PAR
                str_title_add   = " ref prior"    #  # for the prediction
)



#### Validate predictions ####
if(li_general_settings$str_case  == "PAR")
{
  source("calculate_validation_stats_case.R")
  
  
  li_val_REML <- calculate_validation_stats (vn_original_signal = li_obs_data_cov_predgrid$spdf_pred_grid_with_cov$CFAPAR_27,
                                             spdf_prediction_REML
  )
  vn_sum_val_stats_REML <- unlist(li_val_REML[-c(1,2,10)])
  
  
  li_val_BAK_ref <- calculate_validation_stats (vn_original_signal = li_obs_data_cov_predgrid$spdf_pred_grid_with_cov$CFAPAR_27,
                                                spdf_prediction_BAK_ref
  )
  vn_sum_val_stats_BAK_ref <- unlist(li_val_BAK_ref[-c(1,2,10)])
  
  
  
  li_val_BAK_uni <- calculate_validation_stats (vn_original_signal = li_obs_data_cov_predgrid$spdf_pred_grid_with_cov$CFAPAR_27,
                                                spdf_prediction_BAK_uni
  )
  vn_sum_val_stats_BAK_uni <- unlist(li_val_BAK_uni[-c(1,2,10)])
  
  
  li_val_MML <- calculate_validation_stats (vn_original_signal = li_obs_data_cov_predgrid$spdf_pred_grid_with_cov$CFAPAR_27,
                                            spdf_prediction_MML
  )
  vn_sum_val_stats_mml <- unlist(li_val_MML[-c(1,2,10)])
  
  
  ## Add baseline
  
  vn_error <- li_obs_data_cov_predgrid$spdf_pred_grid_with_cov$CFAPAR_27 - li_obs_data_cov_predgrid$spdf_pred_grid_with_cov$baseline
  
  ## Prediction validation 
  # Mean error
  n_mean_error <- mean(vn_error)
  
  # Root Mean Squared error
  n_rmse       <- sqrt( mean( vn_error^2) )
  
  df_sum_val <- data.frame(REML = vn_sum_val_stats_REML,
                           MML  = vn_sum_val_stats_mml,
                           BAKuni = vn_sum_val_stats_BAK_uni,
                           BAKref = vn_sum_val_stats_BAK_ref,
                           Baseline = c(n_mean_error, n_rmse, rep(NA, times=5))
                            )
  (df_sum_val) 
  # Contains the numbers mentioned in "Section: 5.1 Synthetic case study: vegetation index data, with validation on point support"
  
}  


#### Compare sd's ####

## The following code is optimised for millet

bo_export_pdf <- FALSE

n_pdf_pointsize <- 14

bo_bw <- TRUE 

if (bo_bw)
{
  
  li_general_settings$li_export$latex_plot_dir     <- "exportbw/"
  
  str_col_REML <- "black"
  str_col_MML  <- "black"
  str_col_Uni  <- "black"
  str_col_Ref  <- "black"
  str_lty_REML <- "dashed"
  str_lty_MML  <- "longdash"
  str_lty_Uni  <- "dotted"
  str_lty_Ref  <- "solid" 
  n_lwd_MML    <- 2
} else
{
  li_general_settings$li_export$latex_plot_dir     <- "exportcolour/"
  
  str_col_REML <- "green"
  str_col_MML  <- "cyan"
  str_col_Uni  <- "blue"
  str_col_Ref  <- "black"
  str_lty_REML <- "dashed"
  str_lty_MML  <- "longdash"
  str_lty_Uni  <- "dotted"
  str_lty_Ref  <- "solid" 
  n_lwd_MML    <- 2  
}


if(bo_export_pdf) 
{
  pdf( 
    paste0(li_general_settings$li_export$latex_plot_dir,"atpk-case-sd-", li_general_settings$str_case, ".pdf"),
    width = 6,
    height = 6,
    pointsize = n_pdf_pointsize
  )
  str_main = ""
} else 
{
  str_main = "Densities of prediction sd"   
}


switch(li_general_settings$str_case,
       "PAR"     = {
         vn_xlim  <- NULL 
         vn_ylim  <- NULL    
         str_xlab <- "sd [-]"
       },
       "millet"  = {
         vn_xlim  <- c(0, 0.25) 
         vn_ylim  <- c(0, 20)                   
         str_xlab <- "sd [ton ha-1]"         
       },
       "sorghum" = {
         vn_xlim <- c(0, 0.4) 
         vn_ylim <- c(0, 25)                   
         str_xlab <- "sd [ton ha-1]"         
       },
       {
         vn_xlim <- NULL 
         vn_ylim <- NULL
         str_xlab <- ""         
       }   
       
)



plot(density(spdf_prediction_BAK_ref$prediction_sd),
     main = str_main,
     xlab = str_xlab,
     xlim = vn_xlim, 
     ylim = vn_ylim,
     col  = str_col_Ref,
     lty  = str_lty_Ref
)

lines(
  density(spdf_prediction_REML$prediction_sd),
  col = str_col_REML,
  lty = str_lty_REML
)  

lines(
  density(spdf_prediction_BAK_uni$prediction_sd),
  col = str_col_Uni,
  lty = str_lty_Uni
)  

lines(
  density(spdf_prediction_MML$prediction_sd),
  col = str_col_MML,
  lty = str_lty_MML,
  lwd =   n_lwd_MML   
)  

grid()


if(bo_export_pdf) 
{
  dev.off()
}  


#### Compare predictions ####

if(bo_export_pdf) 
{
  pdf( 
    paste0(li_general_settings$li_export$latex_plot_dir,"atpk-case-pred-", li_general_settings$str_case, ".pdf"),
    width = 6,
    height = 6,
    pointsize = n_pdf_pointsize
  )
  
  str_main = ""
} else 
{
  str_main = "Densities of prediction"   
}

switch(li_general_settings$str_case,
       "PAR"     = {
         vn_xlim  <- NULL 
         vn_ylim  <- NULL    
         str_xlab <- "pred [-]"
       },
       "millet"  = {
         vn_xlim  <- c(0, 2.0) 
         vn_ylim  <- c(0, 2.5)                   
         str_xlab <- "pred [ton ha-1]"         
       },
       "sorghum" = {
         vn_xlim <- c(0, 2.0) 
         vn_ylim <- c(0, 2.0)                   
         str_xlab <- "pred [ton ha-1]"         
       },
       {
         vn_xlim <- NULL 
         vn_ylim <- NULL
         str_xlab <- ""         
       }   
       
)


plot(density(spdf_prediction_BAK_ref$prediction),
     main = str_main,
     xlab = str_xlab,
     xlim = vn_xlim, 
     ylim = vn_ylim,     
     col  = str_col_Ref,
     lty  = str_lty_Ref
)

lines(
  density(spdf_prediction_REML$prediction),
  col = str_col_REML,
  lty = str_lty_REML
)  

lines(
  density(spdf_prediction_BAK_uni$prediction),
  col = str_col_Uni,
  lty = str_lty_Uni
)  

lines(
  density(spdf_prediction_MML$prediction),
  col = str_col_MML,
  lty = str_lty_MML,
  lwd = n_lwd_MML
  
)  

grid()


par(xpd=TRUE)

legend(
  #x = "topright",
  x = 1.1,
  y = 2.55,
  legend =
    c("REML",
      "MML",
      "Bayesian-uniform",
      "Bayesian-reference"),
  col = c(
    str_col_REML,
    str_col_MML,
    str_col_Uni,
    str_col_Ref
  ),
  lty = c(
    str_lty_REML,
    str_lty_MML,
    str_lty_Uni,
    str_lty_Ref
  ),
  lwd = c(
    1,
    n_lwd_MML,
    1,
    1
  ),
  #pt.cex = c(1.5),
  bg = "white"
)


if(bo_export_pdf) 
{
  dev.off()
}  


