prepare_1D_simulation_data <- function(li_simulation_generate)
  
## Prepares the 1D simulation data, including the signal and covariate
{

  #li_simulation_generate <- li_general_settings$li_simulation_generate
    
  if(is.null(li_simulation_generate$str_covModel))
  {
    li_simulation_generate$str_covModel <- "ExponentialC"
  }

  ## Supporting function
  spatial_covariance <- function(c1, a, h, c0 = 0)
  {
    if (c0 > 0)
    {   
      mn_cov <- ifelse(h==0,
                       c0 + c1,
                       c1 * exp(-h/a)
      ) 
    } else # no nugget
    {
      mn_cov <- c1 * exp(-h/a)  
    }
    return (mn_cov)
  }  
  
  
  
  
   ## Create line  ####   
  vn_s1 <- seq(from       = 0,
               to         = li_simulation_generate$n_of_au,
               length.out = li_simulation_generate$n_of_nodes
               )
  # vn_s1: Vector with the one-dimensional coordinates

  df_data_points<-data.frame(s1 = vn_s1)
  # df_data_points: Dataframe with (at the end of this function) 1D location s1, 
  # area identifier area, covariate x, original point data z.
  
  
  ## Define areas ####
  
  ## Randomly
  if (li_simulation_generate$bo_random_areas)
  {
    i <- 0 # Prevent endless loops
    repeat{
      vn_boundaries <- c(0,
                         sort(
                           sample.int(li_simulation_generate$n_of_nodes,
                                      li_simulation_generate$n_of_observed_areas - 1
                           )
                         ),    # sample random borders between areas
                         length(vn_s1)
      )          # define borders of areas
      i <- i+1
      if (min(diff(vn_boundaries)) >= li_simulation_generate$n_minimal_area_size)
      {   # if the size of all areas is above the threshold, accept this sample
        break()  
      }
      if (i > 1000) # No valid sample can be found within 1000 tries
      {
        stop("li_simulation_generate$n_minimal_area_size too large;", 
             " cannot find sample with all ", li_simulation_generate$n_of_observed_areas, " areas over ", li_simulation_generate$n_minimal_area_size, " points \n")
        
      }
      
    }
    
  } else # Define areas regulary
  {
    vn_boundaries <- round( 
      seq(from       = 0, 
          to         = li_simulation_generate$n_of_nodes,
          length.out = li_simulation_generate$n_of_observed_areas + 1
      )
    )
  }
  
  
  ## Indicate area id ####
  vn_area_ids <- 1:li_simulation_generate$n_of_observed_areas
  # vn_area_ids: Vector with the area identifiers
  
  df_data_points$area <- rep.int(1:li_simulation_generate$n_of_observed_areas,
                                 times=diff(vn_boundaries)
  )

  ## Calculate covariance matrix all nodes ####
  
  mn_D <- abs(outer(vn_s1,vn_s1, FUN="-")) # Distance-matrix of 1D grid
 
  if (li_simulation_generate$str_covModel == "ExponentialC")
  {
    mn_C <- spatial_covariance(c1 = li_simulation_generate$n_c1, 
                               a  = li_simulation_generate$n_a, 
                               h  = mn_D,
                               c0 = li_simulation_generate$n_c0) # Covariance matrix of 1D grid
  } else if(li_simulation_generate$str_covModel == "MaternC")
  {
    
    mn_C <- li_simulation_generate$n_c1*MaternC(h = mn_D,
                                                par = li_simulation_generate$n_a,
                                                nu = li_simulation_generate$n_nu)
  } else
  {
    stop("Please define a valid covModel")
  }
  

  mn_Upper <- chol(mn_C) #Cholesky decomposition of covariance matrix

  vn_N <- rnorm(n=length(vn_s1),0,1) #simulate random numbers from standard normal distribution
  df_data_points$random <- crossprod(mn_Upper,vn_N)
  
  ## Add spatial trend ####
  if (li_simulation_generate$bo_trend_on_coord)
  {
    
    df_data_points$x <- df_data_points$s1
    
    df_data_points$trend <- li_simulation_generate$n_beta_1 + 
      df_data_points$x * li_simulation_generate$n_beta_2
    
    
  } else # randomness in the trend
  {
    mn_C <- spatial_covariance(c1 = li_simulation_generate$n_c1_trend, 
                               a  = li_simulation_generate$n_a_trend, 
                               h  = mn_D,
                               c0 = li_simulation_generate$n_c0_trend) # Covariance matrix of 1D grid
    mn_Upper <- chol(mn_C) #Cholesky decomposition of covariance matrix

        vn_N <- rnorm(n=length(vn_s1),0,1) #simulate random numbers from standard normal distribution
    df_data_points$x <- crossprod(mn_Upper,vn_N)
    
    df_data_points$trend <- li_simulation_generate$n_beta_1 + 
      df_data_points$x * li_simulation_generate$n_beta_2
  }
  
  df_data_points$z     <- df_data_points$trend + df_data_points$random
  
  df_data_points$random <- NULL
  
  
  
  ## Calculate the "observable" area means ####
  ## compute area means of variable of interest (mz) and covariate (mx) ##
  an_mz <- tapply(df_data_points$z,INDEX=df_data_points$area,FUN=mean)
  an_mx <- tapply(df_data_points$x,INDEX=df_data_points$area,FUN=mean)
  df_data_area <- data.frame(mz = an_mz, mx = an_mx)
  #df_data_area: dataframe of "observable" data
  
  # Create prediction points ####
  # we assume that the prediction points are the original signal locations (points)
  df_prediction_points <- df_data_points[,c("s1","x")]
  # df_prediction_points: Dataframe with prediction points locations(s1) and covariate values (x)"

  # Create also grid for calculation of average covariance ####
  
  # First select index for desired elements from vn_s1
  vn_selection_for_rastered_grid <- round( 
    seq.int(
      from=1, 
      to=length(vn_s1), 
      length.out=li_simulation_generate$n_grid_density
    )
  )
  
  vn_rastered_areas  <- vn_s1[vn_selection_for_rastered_grid]
  # Indicates position of spatial raster to calculate mean covariance for prediction
  
  vn_area_ids_for_obs_grid <- df_data_points$area[vn_selection_for_rastered_grid]
  # Indicates the corresponding area in vn_rastered_areas
  
  
  return( list(
    df_data_points                = df_data_points,
    vn_area_ids                    = vn_area_ids,
    df_data_area                  = df_data_area,
    df_prediction_points          = df_prediction_points,
    vn_rastered_areas             = vn_rastered_areas,  
    vn_area_ids_for_obs_grid       = vn_area_ids_for_obs_grid
  )
  )
  
  
}

