
run_single_simulation_several_methods <- function(n_sim_it # Simulation iteration; is used as random seed
                                                  )
## Performs one simulation, predicts using REML, MML and several Bayesian methods 
#  and summarises validation statistics, among others
  
{
    
  # In a parallel environment, packages and supporting files must be loaded again
  n_dimensionality <- li_general_settings$n_dimensionality # 1 or 2
  require(sp)
  require(invgamma)
  require(DEoptim)
  require(statmod)
  if(n_dimensionality==2)
  {
    require(RandomFields)
  }
  
  source( "./bayesian_areal_kriging/averageCov.R") 
  source( "./bayesian_areal_kriging/postParams.R") 
  source( "./bayesian_areal_kriging/refPriora.R") 
  
  ## Different random seed for every simulation
  set.seed(271828 + n_sim_it)
  
  
  
  if(n_dimensionality == 1)
  {    
    source("prepare_1D_simulation_data.R")
    li_simulation_generate <- li_general_settings$li_simulation_generate # settings for generation
    li_simulation_data <- prepare_1D_simulation_data(li_simulation_generate)
  
  } else if(n_dimensionality == 2)
  {
    source("prepare_2D_simulation_data.R")
    li_generated <- prepare_2D_simulation_data(li_general_settings)
    
    df_area_means       <- li_generated$df_area_means
    df_simulation_field <- li_generated$df_simulation_field
    sppoldf_area_means  <- li_generated$sppoldf_area_means
    
    li_simulation_generate <- li_general_settings$li_simulation_generate  
  }
  
  
  if (is.null(li_general_settings$bo_covariate))
  {
     bo_covariate <- TRUE
  }
  else 
  {
      bo_covariate <- li_general_settings$bo_covariate
  }
  
  if(is.null(li_general_settings$bo_skip_gamma_reference)) # Skip inverse gamma and reference part?
  {
    bo_skip_gamma_reference <- FALSE 
  } else
  {
    bo_skip_gamma_reference <- li_general_settings$bo_skip_gamma_reference
  }
  

  
  if(n_dimensionality == 1)
  {    
    source("create_covariance_objects_1D_sim.R")
    create_covariance_objects_1D_sim(li_simulation_data)  # objects are in global scope
  } else if(n_dimensionality == 2)
  {
    
    ## Prediction points equal generated points 
    
    sp_pred_grid <- df_simulation_field[,
                                        c("x", "y", "area")]; coordinates(sp_pred_grid) <- ~x+y
    
    ## For discretisation
    sp_disc_grid <- df_simulation_field[df_simulation_field$area>0,
                                        c("x", "y", "area")]; coordinates(sp_disc_grid) <- ~x+y
    
    #sp_pred_grid <- sp_disc_grid #For later validation
    
    names(sp_pred_grid) <- "REG_MAP_ID"
    names(sp_disc_grid) <- "REG_MAP_ID"
    
    
    str_prj_planar <<- ""
   
   source("create_covariance_objects.R")
    create_covariance_objects(spdf_pred_grid_with_cov_plan = sp_disc_grid, 
                              sp_pred_grid_plan = sp_pred_grid ) # objects are in global scope
  }
  
  
  
  


  
  if (is.null(li_general_settings$lu_bounds))
  {
    li_general_settings$lu_bounds <- c(10,300)
  }
  
  
  
  #### Data preparation ####
  
  if(n_dimensionality == 1)
  {    
    df_data_area            <- li_simulation_data$df_data_area
    df_data_points          <- li_simulation_data$df_data_points 
    bo_remove_area_0        <- FALSE   # parameter for calculate_validation_stats
  } else if (n_dimensionality == 2)
  {
    # To connect pieces of code from different origine, the same data 
    # has to be stored in different objects
    df_data_area            <- li_generated$df_area_means
    names(df_data_area)     <- c("area_id", "mz", "mx")
    
    df_data_points          <- li_generated$df_simulation_field
    df_data_points$x        <- NULL # remove coordinates
    df_data_points$y        <- NULL
    names(df_data_points)   <- c("x", "z", "area")
    
    li_simulation_data <- list()
    li_simulation_data$df_data_area   <- df_data_area 
    li_simulation_data$df_data_points <- df_data_points  
    
    bo_remove_area_0        <- TRUE   # parameter for calculate_validation_stats,
                                      # in our 2D sim there are a few more 
                                      # predictions than "signal" point locations
    
  }
  
  
  
  
  #### REML ####
  
  li_settings_REML <- list()
  
  li_settings_REML$vn_a_bounds      <- li_general_settings$lu_bounds 
  li_settings_REML$n_max_iter       <- 20    
  
  
  if(bo_covariate)
  {
    lm_bestmodel <- lm(mz ~ mx, data = df_data_area)
    mn_x0 <- cbind(1, df_data_points$x)
  } else # no covariates, only mean
  {
    lm_bestmodel <- lm(mz ~ 1, data = df_data_area)
    mn_x0 <- matrix(1, ncol = 1, nrow = dim(df_data_points)[1])
  }
  
  
  #
  
  source("REML_core.R")
  set.seed(271828) # REML optimalisation uses some randomness
  li_REML_results <- REML_core(lm_bestmodel = lm_bestmodel,
                               vn_zbar      = df_data_area$mz,
                               li_settings_REML = li_settings_REML,
                               mn_x0 = mn_x0
  )
  
  source("calculate_validation_stats.R")  
  li_val_stats_REML <- 
    calculate_validation_stats(
      li_simulation_data   = li_simulation_data,
      vn_z_pred            = li_REML_results$vn_pred ,
      vn_z_pred_sd         = li_REML_results$vn_sd,
      bo_remove_area_0     = bo_remove_area_0
    )   
  
  
  ## Extreme values for a? ####
  n_tol <- diff(range(li_settings_REML$vn_a_bounds))/500
  bo_a_min <- (li_REML_results$n_a < min(li_settings_REML$vn_a_bounds) + n_tol)
  bo_a_max <- (li_REML_results$n_a > max(li_settings_REML$vn_a_bounds) - n_tol )
  
  
  
  vn_validation_stats_REML <- 
    c(unlist(li_val_stats_REML[3:9]), li_REML_results$n_a, NA, bo_a_min, bo_a_max, li_REML_results$n_c1)
  
    names(vn_validation_stats_REML) <- c("ME",     # mean error
                                         "RMSE",   # root mean squared error
                                         "FIR1",   # fraction inside region with sd = 1
                                         "FIR2", 
                                         "meanStSE",   # mean of standardised squared error
                                         "medianStSE", # median of standardised squared error
                                         "maxMPP",     # maximal found error concerning mass preserving property 
                                         "a",          # estimate or mode for a
                                         "cia_sa",     # is credible interval around a used in simulations? 
                                         "e_a_min",    # estimated a or mode a is extreme to min
                                         "e_a_max",    # estimated a or mode a is extreme to max
                                         "c1"          # estimate or mode for c1
                                         )        
  
  df_results_single_simulation_several_methods <- as.data.frame( 
                                                   rbind(REML = vn_validation_stats_REML) 
                                                   )
  

  
  #### Bayesian areal kriging (BAK) load functions & defaults####
  source("collect_summarise_validation_and_Bay_inference.R")
  
  
  n_est_c1 <- df_results_single_simulation_several_methods["REML", "c1"]
 
  priorc1Pars      <<-  c(0, 0)
  #c1Vec <-  1  #We are not interested in the marginal posterior of c1 or beta
  
  c1Vec <- exp(
        seq(log(n_est_c1/10), log(n_est_c1*10), length.out = 100)
        )
  

  if(bo_covariate)
  {
    Xc   <- cbind(1,
                  df_data_area$mx)
    ma_covsGrid <- matrix(df_data_points$x, ncol = 1)
    betaMat <- matrix(0, nrow=2, ncol = 2)
  } else
  {
    Xc   <- matrix(1, 
                   ncol =1, 
                   nrow = length(df_data_area$mz) )
    ma_covsGrid <- matrix(NA, ncol = 1, nrow = length(df_data_points$x)) 
    # postc1abetaz0 needs a matrix to predict, even if all elements are NA
    betaMat <- matrix(0, nrow=1, ncol = 2)    
  }
  

  

  #### PAB uniform ####
  prioraType      <<- 'uni'
  #prioraPars      <<- c(0, 301)
  
  prioraPars      <<- c(max(li_general_settings$lu_bounds[1]-0.05, 0.0001),  
                        li_general_settings$lu_bounds[2]+ 0.05)
  
  # aVec  <- c(
  #   seq(0.05 ,  2.25 , 0.2),
  #   seq(3 ,  50 , 1),
  #   seq(55,  300 , 5)
  # )
  
  aVec  <- exp(
            seq(from = log(li_general_settings$lu_bounds[1]),
               to = log(li_general_settings$lu_bounds[2]),
               length.out = 100)
            )
  # more gridpoints with lower values

  
  li_BAK_results <- postc1abetaz0(z = df_data_area$mz , 
                                 X = Xc , 
                                 aVec = aVec , 
                                 c1Vec = c1Vec , 
                                 betaMat = betaMat,
                                 covsGrid =  ma_covsGrid
  )
  
  
  
  # if ( any(li_BAK_results$v0<0) )
  # {
  #   cat("\n Warning: at least one negative prediction variance! \n")
  # }

  df_results_single_simulation_several_methods <- 
    rbind(
      df_results_single_simulation_several_methods,
      Uniform = collect_summarise_validation_and_Bay_inference(li_BAK_results, 
                                    li_simulation_data,
                                    li_simulation_generate,
                                    bo_remove_area_0
                                    )
    ) 
  
  
  ## for later use with MML: the mode of posterior(a), resulting from uniform prior
  n_mode_a <- df_results_single_simulation_several_methods["Uniform", "a"]
  
  
  
  #### PAB inv gamma ####
  
  if(!bo_skip_gamma_reference)
  {
  
    #For inverse gamma and reference prior, we set the range parameter grid to a much larger range,
    #   upto 2000
    aVec  <- c(
      seq(0.05 ,  2.25 , 0.2),
      seq(3 ,  50 , 1),
      seq(55,  300 , 5),
      seq(350,  2000 , 50)
    )
    prioraType           <<-  'invgamma'
    prioraPars           <<-  c(11 , 600)
  
  
  
    #curve(dinvgamma(x,shape=11, rate = 600 ), to=200)
  
    li_BAK_results <- postc1abetaz0(z = df_data_area$mz ,
                                        X = Xc ,
                                        aVec = aVec ,
                                        c1Vec = c1Vec ,
                                        betaMat = betaMat,
                                        covsGrid =  ma_covsGrid
    )
  
  
    df_results_single_simulation_several_methods <-
      rbind(
        df_results_single_simulation_several_methods,
        InvGam = collect_summarise_validation_and_Bay_inference(li_BAK_results,
                                     li_simulation_data,
                                     li_simulation_generate,
                                     bo_remove_area_0)
      )

  } else
  {
    df_results_single_simulation_several_methods <-
      rbind(
        df_results_single_simulation_several_methods,
        InvGam = NA
      )
    
  }
  

  
  #### PAB reference ####
  if(!bo_skip_gamma_reference)
  {
    
  
  prioraType           <<-  'reference'
  prioraPars           <<-  NA

  li_BAK_results <- postc1abetaz0(z = df_data_area$mz ,
                                  X = Xc ,
                                  aVec = aVec ,
                                  c1Vec = c1Vec ,
                                  betaMat = betaMat,
                                  covsGrid =  ma_covsGrid
  )


  df_results_single_simulation_several_methods <-
    rbind(
      df_results_single_simulation_several_methods,
      Reference = collect_summarise_validation_and_Bay_inference(li_BAK_results,
                                      li_simulation_data,
                                      li_simulation_generate,
                                      bo_remove_area_0)
    )


  } else
  {
    
    df_results_single_simulation_several_methods <-
      rbind(
        df_results_single_simulation_several_methods,
        Reference = NA
      )    
    
    
  }
  

  #### PAB fixed based on mode uniform (Maximal Marginal Likelihood) ####
  
 
  prioraType           <<-  'uni'
  prioraPars           <<-  c(n_mode_a-0.1, 
                             n_mode_a+0.1)
  
  aVec                 <-  c(n_mode_a - 0.001,
                             n_mode_a,
                             n_mode_a + 0.001
                           )
  
  
  li_BAK_results <- postc1abetaz0(z = df_data_area$mz , 
                                  X = Xc , 
                                  aVec = aVec , 
                                  c1Vec = c1Vec , 
                                  betaMat = betaMat,
                                  covsGrid =  ma_covsGrid
  )
  
  li_val_stats_BAK  <- 
    calculate_validation_stats(
      li_simulation_data = li_simulation_data,
      vn_z_pred = li_BAK_results$z0,
      vn_z_pred_sd = sqrt(ifelse(li_BAK_results$v0<0,0,li_BAK_results$v0)),
      bo_remove_area_0 = bo_remove_area_0
    )  
  

  vn_validation_stats_BAK <- c( unlist(li_val_stats_BAK[3:9]),
                                df_results_single_simulation_several_methods["Uniform", c("a")] , 
                                NA,
                                df_results_single_simulation_several_methods["Uniform", c("e_a_min")],
                                df_results_single_simulation_several_methods["Uniform", c("e_a_max")],
                                NA
                              )
  
  df_results_single_simulation_several_methods <- 
    rbind(
      df_results_single_simulation_several_methods,
      MML = c(vn_validation_stats_BAK)
    ) 
  
  
  
  #### Baseline ####
  
  
  if(n_dimensionality == 1)
  {    
    vn_z_pred <- as.numeric(
      li_simulation_data$df_data_area$mz[li_simulation_data$df_data_points$area]
    )
    vn_z_pred_sd <- rep(
      x = sd(li_simulation_data$df_data_area$mz),
      times = length(vn_z_pred)
    )
    
    li_val_stats_baseline  <- 
      calculate_validation_stats(
        li_simulation_data = li_simulation_data,
        vn_z_pred = vn_z_pred,
        vn_z_pred_sd = NA
      )

  } else if(n_dimensionality == 2)
  {
    df_data_points_tmp <- li_simulation_data$df_data_points[li_simulation_data$df_data_points$area > 0, ]
    vn_z_pred <- as.numeric(
      li_simulation_data$df_data_area$mz[df_data_points_tmp$area]
    )
    # vn_z_pred_sd <- rep(
    #   x = sd(li_simulation_data$df_data_area$mz),
    #   times = length(vn_z_pred)
    # )
    
    li_val_stats_baseline  <- 
      calculate_validation_stats(
        li_simulation_data = li_simulation_data,
        vn_z_pred = vn_z_pred,
        vn_z_pred_sd = NA,
        bo_remove_area_0 = FALSE,
        bo_remove_points_with_area_0 = TRUE
      )  
  }
 

  vn_validation_stats_baseline <- 
    c( unlist(li_val_stats_baseline[3:9]) ) 
  
  df_results_single_simulation_several_methods <- 
    rbind(
      df_results_single_simulation_several_methods,
      Baseline = c(vn_validation_stats_baseline, NA, NA, NA, NA, NA )
    ) 
  
  ## Polishing the results dataframe ####
  df_results_single_simulation_several_methods$cia_sa <- 
    as.logical(df_results_single_simulation_several_methods$cia_sa)
  
  df_results_single_simulation_several_methods$e_a_min <- 
    as.logical(df_results_single_simulation_several_methods$e_a_min)
  
  df_results_single_simulation_several_methods$e_a_max <- 
    as.logical(df_results_single_simulation_several_methods$e_a_max)
  
  
  return(df_results_single_simulation_several_methods)
}

