show_variogram <- function(li_export, 
                           li_simulation_data,
                           li_BAK_results,
                           li_REML_results,
                           bo_bw = FALSE)

## Plots the experimental variogram, several Bayesian inferenced variogram 
#  models and the REML fitted variogram model (figure 5 in paper)
  
{  
    
  
  vn_seq_for_colours <- (seq(from=0, to=1, length.out=101))
  
  vs_colours <- rgb( red   = 1-vn_seq_for_colours,
                     green = 1-vn_seq_for_colours,
                     blue  = 1-vn_seq_for_colours,
                     alpha = 0.75)
  
  bo_plots_to_latex  <- li_export$bo_plots_to_latex
  str_latex_plot_dir <- li_export$latex_plot_dir
  
  if(bo_bw)
  {
    str_colour_exper_var <- "black"
    str_colour_REML_var  <- "black"
    str_colour_leg_bg    <- "white"
    str_lty_REML         <- "solid" 
  } else
  {
    str_colour_exper_var <- "brown"
    str_colour_REML_var  <- "green"
    str_colour_leg_bg    <- "white" 
    str_lty_REML         <- "solid"  
  }
  
  n_pdf_pointsize <- 12
  
  
  
  if(bo_plots_to_latex)
  {
    pdf(file = paste0(str_latex_plot_dir, "variogram-reml-density.pdf"),
        width = 8,
        height = 6,
        pointsize = n_pdf_pointsize
    )
    
  }
  
  ### Create experimental variogram
  vari <- variog(coords = cbind(li_simulation_data$df_data_points$s1, 0), 
                 data = li_simulation_data$df_data_points$z-li_simulation_data$df_data_points$trend,
                 max.dist = 227
  )
  #clip(0, 220, 0, 10)
  plot(vari, 
       type = "b", 
       main = "", 
       lwd = 2, 
       xlim = c(0,220),
       ylim = c(0,10),
       xlab = "",
       ylab = "",     
       col = str_colour_exper_var)  
  
  mtext("Distance [au]", 
        side = 1, 
        line = 2 ) 
  
  mtext("Semivariance", 
        side = 2, 
        line = 2 )   
  
  
  ### Define matrix of densities; rows are c1, columns are a
  ma_fpc1a <- li_BAK_results$fpc1a  
  
  ### To plot from lower to higher density, we sort, and also take out the many lowest densities
  vn_dens_sorted <- sort(unique(ma_fpc1a))
  vn_dens_sorted <- vn_dens_sorted[vn_dens_sorted>.01]
  # Note that we assume that every density > 0.01 is unique
  
  ## We are going to plot the variogram lines from low to high density;
  ## to be able to find the corresponding c1 and a, we must be able to find 
  ## the corresponding row number and column number of a density in the source matrix,
  ## with this supporting function:
  match_row_column <- function(value_to_be_mached, matrix )
  {  
    
    n_r <- nrow(matrix)
    n_i <- match(value_to_be_mached, matrix)
    
    n_ci <- ceiling(n_i/n_r)
    n_ri <- n_i - (n_ci-1) * n_r
    
    return( c(n_ri, n_ci) )
  }
  
  ## range for colour
  vn_range_dens <- range(vn_dens_sorted)
  
  ## Loop over every density, eventually thinned;
  ## can take a few seconds, even minutes
  for(n_dens in vn_dens_sorted[c(TRUE, rep(FALSE, 25))]  )
  {
    
    vn_indices <- match_row_column(n_dens, ma_fpc1a)
    
    
    n_c1 <- li_BAK_results$c1Vec[vn_indices[1]]
    n_a  <- li_BAK_results$aVec[vn_indices[2]]
    
    
    vgm_BAK <- vgm(psill = n_c1,
                   model = "Exp",
                   range = n_a
    )
    
    lines(variogramLine(vgm_BAK, maxdist=220),
          col = vs_colours[round(
            rescale(x = n_dens,
                    from = vn_range_dens,
                    to = c(1,100)
            ))+1],
          lwd = 1
    )
    
    
  }
  
  
  
  ### Add REML
  
  vgm_01 <- vgm(psill = li_REML_results$n_c1,
                model = "Exp",
                range = li_REML_results$n_a
  )
  
  lines(variogramLine(vgm_01, maxdist=220),
        col = str_colour_REML_var,
        lwd = 4,
        lty = str_lty_REML
  )
  
  
  ### Redraw experimental variogram for visibility
  lines(vari,      
        type = "b", 
        main = "", 
        lwd = 2, 
        col = str_colour_exper_var)
  
  
  
  ### Legend etc
  
  
  legend(#xpd = TRUE, # to enable printing outside plot
    x = "topleft", 
    bg = "white",
    legend = c("Experimental variogram",
               "Bayesian-Uniform inferenced variogram models",
               "REML fitted variogram model"
    ),
    col = c( NA,
             NA, #str_colour_exper_var, 
             str_colour_REML_var),
    pch    = c(NA, NA, NA),
    lty    = c(NA, NA, str_lty_REML),
    lwd    = c(NA, NA, 4)
  )
  
  if(bo_plots_to_latex)
  {
    lines(x = c(-6, 3),
          y = c(9.85, 9.85),      
          type = "b", 
          lwd = 2, 
          col = str_colour_exper_var)  
    
    lines(x = c(-7, 5),
          y = c(9.4, 9.4),
          col = "black")
  
    lines(x = c(-7, 5),
          y = c(9.35, 9.35),
          col = "darkgray")
    
  
    
  
  }  else # plot inside GUI
  {
    # Might need some finetuning depending on plotting area
    lines(x = c(-6, 3),
          y = c(9.85, 9.85),      
          type = "b", 
          lwd = 2, 
          col = str_colour_exper_var)  
    
    lines(x = c(-7, 5),
          y = c(9.4, 9.4),
          col = "black")
    
    lines(x = c(-7, 5),
          y = c(9.35, 9.35),
          col = "darkgray")
    
  }
    
    
  box()
  
  if(bo_plots_to_latex)
  {
    dev.off()
  }

}